@@ -10,8 +10,8 @@ from freqtrade.commands.arguments import Arguments
 from freqtrade.commands.build_config_commands import start_new_config
 from freqtrade.commands.data_commands import (start_convert_data, start_download_data,
                                               start_list_data)
-from freqtrade.commands.deploy_commands import (start_create_userdir, start_new_hyperopt,
-                                                start_new_strategy)
+from freqtrade.commands.deploy_commands import (start_create_userdir, start_install_ui,
+                                                start_new_hyperopt, start_new_strategy)
 from freqtrade.commands.hyperopt_commands import start_hyperopt_list, start_hyperopt_show
 from freqtrade.commands.list_commands import (start_list_exchanges, start_list_hyperopts,
                                               start_list_markets, start_list_strategies,
