@@ -2,23 +2,35 @@
 Helpers when analyzing backtest data
 """
 import logging
-from datetime import timezone
 from pathlib import Path
-from typing import Any, Dict, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
 
 from freqtrade.constants import LAST_BT_RESULT_FN
 from freqtrade.misc import json_load
-from freqtrade.persistence import Trade, init_db
+from freqtrade.persistence import LocalTrade, Trade, init_db
 
 
 logger = logging.getLogger(__name__)
 
-# must align with columns in backtest.py
-BT_DATA_COLUMNS = ["pair", "profit_percent", "open_date", "close_date", "index", "trade_duration",
-                   "open_rate", "close_rate", "open_at_end", "sell_reason"]
+# Old format - maybe remove?
+BT_DATA_COLUMNS_OLD = ["pair", "profit_percent", "open_date", "close_date", "index",
+                       "trade_duration", "open_rate", "close_rate", "open_at_end", "sell_reason"]
+
+# Mid-term format, crated by BacktestResult Named Tuple
+BT_DATA_COLUMNS_MID = ['pair', 'profit_percent', 'open_date', 'close_date', 'trade_duration',
+                       'open_rate', 'close_rate', 'open_at_end', 'sell_reason', 'fee_open',
+                       'fee_close', 'amount', 'profit_abs', 'profit_ratio']
+
+# Newest format
+BT_DATA_COLUMNS = ['pair', 'stake_amount', 'amount', 'open_date', 'close_date',
+                   'open_rate', 'close_rate',
+                   'fee_open', 'fee_close', 'trade_duration',
+                   'profit_ratio', 'profit_abs', 'sell_reason',
+                   'initial_stop_loss_abs', 'initial_stop_loss_ratio', 'stop_loss_abs',
+                   'stop_loss_ratio', 'min_rate', 'max_rate', 'is_open', ]
 
 
 def get_latest_optimize_filename(directory: Union[Path, str], variant: str) -> str:
@@ -154,7 +166,7 @@ def load_backtest_data(filename: Union[Path, str], strategy: Optional[str] = Non
                                           )
     else:
         # old format - only with lists.
-        df = pd.DataFrame(data, columns=BT_DATA_COLUMNS)
+        df = pd.DataFrame(data, columns=BT_DATA_COLUMNS_OLD)
 
         df['open_date'] = pd.to_datetime(df['open_date'],
                                          unit='s',
@@ -166,7 +178,10 @@ def load_backtest_data(filename: Union[Path, str], strategy: Optional[str] = Non
                                           utc=True,
                                           infer_datetime_format=True
                                           )
+        # Create compatibility with new format
         df['profit_abs'] = df['close_rate'] - df['open_rate']
+    if 'profit_ratio' not in df.columns:
+        df['profit_ratio'] = df['profit_percent']
     df = df.sort_values("open_date").reset_index(drop=True)
     return df
 
@@ -209,6 +224,20 @@ def evaluate_result_multi(results: pd.DataFrame, timeframe: str,
     return df_final[df_final['open_trades'] > max_open_trades]
 
 
+def trade_list_to_dataframe(trades: List[LocalTrade]) -> pd.DataFrame:
+    """
+    Convert list of Trade objects to pandas Dataframe
+    :param trades: List of trade objects
+    :return: Dataframe with BT_DATA_COLUMNS
+    """
+    df = pd.DataFrame.from_records([t.to_json() for t in trades], columns=BT_DATA_COLUMNS)
+    if len(df) > 0:
+        df.loc[:, 'close_date'] = pd.to_datetime(df['close_date'], utc=True)
+        df.loc[:, 'open_date'] = pd.to_datetime(df['open_date'], utc=True)
+        df.loc[:, 'close_rate'] = df['close_rate'].astype('float64')
+    return df
+
+
 def load_trades_from_db(db_url: str, strategy: Optional[str] = None) -> pd.DataFrame:
     """
     Load trades from a DB (using dburl)
@@ -219,36 +248,10 @@ def load_trades_from_db(db_url: str, strategy: Optional[str] = None) -> pd.DataF
     """
     init_db(db_url, clean_open_orders=False)
 
-    columns = ["pair", "open_date", "close_date", "profit", "profit_percent",
-               "open_rate", "close_rate", "amount", "trade_duration", "sell_reason",
-               "fee_open", "fee_close", "open_rate_requested", "close_rate_requested",
-               "stake_amount", "max_rate", "min_rate", "id", "exchange",
-               "stop_loss", "initial_stop_loss", "strategy", "timeframe"]
-
     filters = []
     if strategy:
         filters.append(Trade.strategy == strategy)
-
-    trades = pd.DataFrame([(t.pair,
-                            t.open_date.replace(tzinfo=timezone.utc),
-                            t.close_date.replace(tzinfo=timezone.utc) if t.close_date else None,
-                            t.calc_profit(), t.calc_profit_ratio(),
-                            t.open_rate, t.close_rate, t.amount,
-                            (round((t.close_date.timestamp() - t.open_date.timestamp()) / 60, 2)
-                             if t.close_date else None),
-                            t.sell_reason,
-                            t.fee_open, t.fee_close,
-                            t.open_rate_requested,
-                            t.close_rate_requested,
-                            t.stake_amount,
-                            t.max_rate,
-                            t.min_rate,
-                            t.id, t.exchange,
-                            t.stop_loss, t.initial_stop_loss,
-                            t.strategy, t.timeframe
-                            )
-                           for t in Trade.get_trades(filters).all()],
-                          columns=columns)
+    trades = trade_list_to_dataframe(Trade.get_trades(filters).all())
 
     return trades
 
@@ -309,7 +312,7 @@ def calculate_market_change(data: Dict[str, pd.DataFrame], column: str = "close"
         end = df[column].dropna().iloc[-1]
         tmp_means.append((end - start) / start)
 
-    return np.mean(tmp_means)
+    return float(np.mean(tmp_means))
 
 
 def combine_dataframes_with_mean(data: Dict[str, pd.DataFrame],
@@ -334,7 +337,7 @@ def create_cum_profit(df: pd.DataFrame, trades: pd.DataFrame, col_name: str,
     """
     Adds a column `col_name` with the cumulative profit for the given trades array.
     :param df: DataFrame with date index
-    :param trades: DataFrame containing trades (requires columns close_date and profit_percent)
+    :param trades: DataFrame containing trades (requires columns close_date and profit_abs)
     :param col_name: Column name that will be assigned the results
     :param timeframe: Timeframe used during the operations
     :return: Returns df with one additional column, col_name, containing the cumulative profit.
@@ -346,8 +349,8 @@ def create_cum_profit(df: pd.DataFrame, trades: pd.DataFrame, col_name: str,
     timeframe_minutes = timeframe_to_minutes(timeframe)
     # Resample to timeframe to make sure trades match candles
     _trades_sum = trades.resample(f'{timeframe_minutes}min', on='close_date'
-                                  )[['profit_percent']].sum()
-    df.loc[:, col_name] = _trades_sum.cumsum()
+                                  )[['profit_abs']].sum()
+    df.loc[:, col_name] = _trades_sum['profit_abs'].cumsum()
     # Set first value to 0
     df.loc[df.iloc[0].name, col_name] = 0
     # FFill to get continuous
@@ -356,14 +359,15 @@ def create_cum_profit(df: pd.DataFrame, trades: pd.DataFrame, col_name: str,
 
 
 def calculate_max_drawdown(trades: pd.DataFrame, *, date_col: str = 'close_date',
-                           value_col: str = 'profit_percent'
-                           ) -> Tuple[float, pd.Timestamp, pd.Timestamp]:
+                           value_col: str = 'profit_ratio'
+                           ) -> Tuple[float, pd.Timestamp, pd.Timestamp, float, float]:
     """
     Calculate max drawdown and the corresponding close dates
-    :param trades: DataFrame containing trades (requires columns close_date and profit_percent)
+    :param trades: DataFrame containing trades (requires columns close_date and profit_ratio)
     :param date_col: Column in DataFrame to use for dates (defaults to 'close_date')
-    :param value_col: Column in DataFrame to use for values (defaults to 'profit_percent')
-    :return: Tuple (float, highdate, lowdate) with absolute max drawdown, high and low time
+    :param value_col: Column in DataFrame to use for values (defaults to 'profit_ratio')
+    :return: Tuple (float, highdate, lowdate, highvalue, lowvalue) with absolute max drawdown,
+             high and low time and high and low value.
     :raise: ValueError if trade-dataframe was found empty.
     """
     if len(trades) == 0:
@@ -379,4 +383,26 @@ def calculate_max_drawdown(trades: pd.DataFrame, *, date_col: str = 'close_date'
         raise ValueError("No losing trade, therefore no drawdown.")
     high_date = profit_results.loc[max_drawdown_df.iloc[:idxmin]['high_value'].idxmax(), date_col]
     low_date = profit_results.loc[idxmin, date_col]
-    return abs(min(max_drawdown_df['drawdown'])), high_date, low_date
+    high_val = max_drawdown_df.loc[max_drawdown_df.iloc[:idxmin]
+                                   ['high_value'].idxmax(), 'cumulative']
+    low_val = max_drawdown_df.loc[idxmin, 'cumulative']
+    return abs(min(max_drawdown_df['drawdown'])), high_date, low_date, high_val, low_val
+
+
+def calculate_csum(trades: pd.DataFrame, starting_balance: float = 0) -> Tuple[float, float]:
+    """
+    Calculate min/max cumsum of trades, to show if the wallet/stake amount ratio is sane
+    :param trades: DataFrame containing trades (requires columns close_date and profit_percent)
+    :param starting_balance: Add starting balance to results, to show the wallets high / low points
+    :return: Tuple (float, float) with cumsum of profit_abs
+    :raise: ValueError if trade-dataframe was found empty.
+    """
+    if len(trades) == 0:
+        raise ValueError("Trade dataframe empty.")
+
+    csum_df = pd.DataFrame()
+    csum_df['sum'] = trades['profit_abs'].cumsum()
+    csum_min = csum_df['sum'].min() + starting_balance
+    csum_max = csum_df['sum'].max() + starting_balance
+
+    return csum_min, csum_max
