@@ -86,8 +86,12 @@ class JsonDataHandler(IDataHandler):
         filename = self._pair_data_filename(self._datadir, pair, timeframe)
         if not filename.exists():
             return DataFrame(columns=self._columns)
-        pairdata = read_json(filename, orient='values')
-        pairdata.columns = self._columns
+        try:
+            pairdata = read_json(filename, orient='values')
+            pairdata.columns = self._columns
+        except ValueError:
+            logger.error(f"Could not load data for {pair}.")
+            return DataFrame(columns=self._columns)
         pairdata = pairdata.astype(dtype={'open': 'float', 'high': 'float',
                                           'low': 'float', 'close': 'float', 'volume': 'float'})
         pairdata['date'] = to_datetime(pairdata['date'],
