@@ -6,11 +6,13 @@ from freqtrade.exchange.exchange import Exchange
 from freqtrade.exchange.bibox import Bibox
 from freqtrade.exchange.binance import Binance
 from freqtrade.exchange.bittrex import Bittrex
+from freqtrade.exchange.bybit import Bybit
 from freqtrade.exchange.exchange import (available_exchanges, ccxt_exchanges,
-                                         get_exchange_bad_reason, is_exchange_bad,
                                          is_exchange_known_ccxt, is_exchange_officially_supported,
                                          market_is_active, timeframe_to_minutes, timeframe_to_msecs,
                                          timeframe_to_next_date, timeframe_to_prev_date,
-                                         timeframe_to_seconds)
+                                         timeframe_to_seconds, validate_exchange,
+                                         validate_exchanges)
 from freqtrade.exchange.ftx import Ftx
 from freqtrade.exchange.kraken import Kraken
+from freqtrade.exchange.kucoin import Kucoin
