@@ -18,6 +18,7 @@ class Kraken(Exchange):
     _params: Dict = {"trading_agreement": "agree"}
     _ft_has: Dict = {
         "stoploss_on_exchange": True,
+        "ohlcv_candle_limit": 720,
         "trades_pagination": "id",
         "trades_pagination_arg": "since",
     }
@@ -47,7 +48,7 @@ class Kraken(Exchange):
 
             orders = self._api.fetch_open_orders()
             order_list = [(x["symbol"].split("/")[0 if x["side"] == "sell" else 1],
-                           x["remaining"],
+                           x["remaining"] if x["side"] == "sell" else x["remaining"] * x["price"],
                            # Don't remove the below comment, this can be important for debuggung
                            # x["side"], x["amount"],
                            ) for x in orders]
@@ -91,7 +92,7 @@ class Kraken(Exchange):
         stop_price = self.price_to_precision(pair, stop_price)
 
         if self._config['dry_run']:
-            dry_order = self.dry_run_order(
+            dry_order = self.create_dry_run_order(
                 pair, ordertype, "sell", amount, stop_price)
             return dry_order
 
