@@ -13,6 +13,7 @@ from freqtrade.data.history import get_timerange, load_data
 from freqtrade.exceptions import OperationalException
 from freqtrade.exchange import timeframe_to_prev_date, timeframe_to_seconds
 from freqtrade.misc import pair_to_filename
+from freqtrade.plugins.pairlist.pairlist_helpers import expand_pairlist
 from freqtrade.resolvers import ExchangeResolver, StrategyResolver
 from freqtrade.strategy import IStrategy
 
@@ -29,16 +30,16 @@ except ImportError:
     exit(1)
 
 
-def init_plotscript(config, startup_candles: int = 0):
+def init_plotscript(config, markets: List, startup_candles: int = 0):
     """
     Initialize objects needed for plotting
     :return: Dict with candle (OHLCV) data, trades and pairs
     """
 
     if "pairs" in config:
-        pairs = config['pairs']
+        pairs = expand_pairlist(config['pairs'], markets)
     else:
-        pairs = config['exchange']['pair_whitelist']
+        pairs = expand_pairlist(config['exchange']['pair_whitelist'], markets)
 
     # Set timerange to use
     timerange = TimeRange.parse_timerange(config.get('timerange'))
@@ -52,7 +53,7 @@ def init_plotscript(config, startup_candles: int = 0):
         data_format=config.get('dataformat_ohlcv', 'json'),
     )
 
-    if startup_candles:
+    if startup_candles and data:
         min_date, max_date = get_timerange(data)
         logger.info(f"Loading data from {min_date} to {max_date}")
         timerange.adjust_start_if_necessary(timeframe_to_seconds(config.get('timeframe', '5m')),
@@ -66,14 +67,16 @@ def init_plotscript(config, startup_candles: int = 0):
         if not filename.is_dir() and not filename.is_file():
             logger.warning("Backtest file is missing skipping trades.")
             no_trades = True
-
-    trades = load_trades(
-        config['trade_source'],
-        db_url=config.get('db_url'),
-        exportfilename=filename,
-        no_trades=no_trades,
-        strategy=config.get('strategy'),
-    )
+    try:
+        trades = load_trades(
+            config['trade_source'],
+            db_url=config.get('db_url'),
+            exportfilename=filename,
+            no_trades=no_trades,
+            strategy=config.get('strategy'),
+        )
+    except ValueError as e:
+        raise OperationalException(e) from e
     trades = trim_dataframe(trades, timerange, 'open_date')
 
     return {"ohlcv": data,
@@ -142,7 +145,7 @@ def add_max_drawdown(fig, row, trades: pd.DataFrame, df_comb: pd.DataFrame,
     Add scatter points indicating max drawdown
     """
     try:
-        max_drawdown, highdate, lowdate = calculate_max_drawdown(trades)
+        max_drawdown, highdate, lowdate, _, _ = calculate_max_drawdown(trades)
 
         drawdown = go.Scatter(
             x=[highdate, lowdate],
@@ -174,10 +177,10 @@ def plot_trades(fig, trades: pd.DataFrame) -> make_subplots:
     # Trades can be empty
     if trades is not None and len(trades) > 0:
         # Create description for sell summarizing the trade
-        trades['desc'] = trades.apply(lambda row: f"{round(row['profit_percent'] * 100, 1)}%, "
+        trades['desc'] = trades.apply(lambda row: f"{round(row['profit_ratio'] * 100, 1)}%, "
                                                   f"{row['sell_reason']}, "
                                                   f"{row['trade_duration']} min",
-                                                  axis=1)
+                                      axis=1)
         trade_buys = go.Scatter(
             x=trades["open_date"],
             y=trades["open_rate"],
@@ -194,9 +197,9 @@ def plot_trades(fig, trades: pd.DataFrame) -> make_subplots:
         )
 
         trade_sells = go.Scatter(
-            x=trades.loc[trades['profit_percent'] > 0, "close_date"],
-            y=trades.loc[trades['profit_percent'] > 0, "close_rate"],
-            text=trades.loc[trades['profit_percent'] > 0, "desc"],
+            x=trades.loc[trades['profit_ratio'] > 0, "close_date"],
+            y=trades.loc[trades['profit_ratio'] > 0, "close_rate"],
+            text=trades.loc[trades['profit_ratio'] > 0, "desc"],
             mode='markers',
             name='Sell - Profit',
             marker=dict(
@@ -207,9 +210,9 @@ def plot_trades(fig, trades: pd.DataFrame) -> make_subplots:
             )
         )
         trade_sells_loss = go.Scatter(
-            x=trades.loc[trades['profit_percent'] <= 0, "close_date"],
-            y=trades.loc[trades['profit_percent'] <= 0, "close_rate"],
-            text=trades.loc[trades['profit_percent'] <= 0, "desc"],
+            x=trades.loc[trades['profit_ratio'] <= 0, "close_date"],
+            y=trades.loc[trades['profit_ratio'] <= 0, "close_rate"],
+            text=trades.loc[trades['profit_ratio'] <= 0, "desc"],
             mode='markers',
             name='Sell - Loss',
             marker=dict(
@@ -263,6 +266,65 @@ def create_plotconfig(indicators1: List[str], indicators2: List[str],
     return plot_config
 
 
+def plot_area(fig, row: int, data: pd.DataFrame, indicator_a: str,
+              indicator_b: str, label: str = "",
+              fill_color: str = "rgba(0,176,246,0.2)") -> make_subplots:
+    """ Creates a plot for the area between two traces and adds it to fig.
+    :param fig: Plot figure to append to
+    :param row: row number for this plot
+    :param data: candlestick DataFrame
+    :param indicator_a: indicator name as populated in stragetie
+    :param indicator_b: indicator name as populated in stragetie
+    :param label: label for the filled area
+    :param fill_color: color to be used for the filled area
+    :return: fig with added  filled_traces plot
+    """
+    if indicator_a in data and indicator_b in data:
+        # make lines invisible to get the area plotted, only.
+        line = {'color': 'rgba(255,255,255,0)'}
+        # TODO: Figure out why scattergl causes problems plotly/plotly.js#2284
+        trace_a = go.Scatter(x=data.date, y=data[indicator_a],
+                             showlegend=False,
+                             line=line)
+        trace_b = go.Scatter(x=data.date, y=data[indicator_b], name=label,
+                             fill="tonexty", fillcolor=fill_color,
+                             line=line)
+        fig.add_trace(trace_a, row, 1)
+        fig.add_trace(trace_b, row, 1)
+    return fig
+
+
+def add_areas(fig, row: int, data: pd.DataFrame, indicators) -> make_subplots:
+    """ Adds all area plots (specified in plot_config) to fig.
+    :param fig: Plot figure to append to
+    :param row: row number for this plot
+    :param data: candlestick DataFrame
+    :param indicators: dict with indicators. ie.: plot_config['main_plot'] or
+                            plot_config['subplots'][subplot_label]
+    :return: fig with added  filled_traces plot
+    """
+    for indicator, ind_conf in indicators.items():
+        if 'fill_to' in ind_conf:
+            indicator_b = ind_conf['fill_to']
+            if indicator in data and indicator_b in data:
+                label = ind_conf.get('fill_label',
+                                     f'{indicator}<>{indicator_b}')
+                fill_color = ind_conf.get('fill_color', 'rgba(0,176,246,0.2)')
+                fig = plot_area(fig, row, data, indicator, indicator_b,
+                                label=label, fill_color=fill_color)
+            elif indicator not in data:
+                logger.info(
+                    'Indicator "%s" ignored. Reason: This indicator is not '
+                    'found in your strategy.', indicator
+                )
+            elif indicator_b not in data:
+                logger.info(
+                        'fill_to: "%s" ignored. Reason: This indicator is not '
+                        'in your strategy.', indicator_b
+                )
+    return fig
+
+
 def generate_candlestick_graph(pair: str, data: pd.DataFrame, trades: pd.DataFrame = None, *,
                                indicators1: List[str] = [],
                                indicators2: List[str] = [],
@@ -280,7 +342,6 @@ def generate_candlestick_graph(pair: str, data: pd.DataFrame, trades: pd.DataFra
     :return: Plotly figure
     """
     plot_config = create_plotconfig(indicators1, indicators2, plot_config)
-
     rows = 2 + len(plot_config['subplots'])
     row_widths = [1 for _ in plot_config['subplots']]
     # Define the graph
@@ -346,36 +407,20 @@ def generate_candlestick_graph(pair: str, data: pd.DataFrame, trades: pd.DataFra
             fig.add_trace(sells, 1, 1)
         else:
             logger.warning("No sell-signals found.")
-
-    # TODO: Figure out why scattergl causes problems plotly/plotly.js#2284
-    if 'bb_lowerband' in data and 'bb_upperband' in data:
-        bb_lower = go.Scatter(
-            x=data.date,
-            y=data.bb_lowerband,
-            showlegend=False,
-            line={'color': 'rgba(255,255,255,0)'},
-        )
-        bb_upper = go.Scatter(
-            x=data.date,
-            y=data.bb_upperband,
-            name='Bollinger Band',
-            fill="tonexty",
-            fillcolor="rgba(0,176,246,0.2)",
-            line={'color': 'rgba(255,255,255,0)'},
-        )
-        fig.add_trace(bb_lower, 1, 1)
-        fig.add_trace(bb_upper, 1, 1)
-        if ('bb_upperband' in plot_config['main_plot']
-           and 'bb_lowerband' in plot_config['main_plot']):
-            del plot_config['main_plot']['bb_upperband']
-            del plot_config['main_plot']['bb_lowerband']
-
-    # Add indicators to main plot
+    # Add Bollinger Bands
+    fig = plot_area(fig, 1, data, 'bb_lowerband', 'bb_upperband',
+                    label="Bollinger Band")
+    # prevent bb_lower and bb_upper from plotting
+    try:
+        del plot_config['main_plot']['bb_lowerband']
+        del plot_config['main_plot']['bb_upperband']
+    except KeyError:
+        pass
+    # main plot goes to row 1
     fig = add_indicators(fig=fig, row=1, indicators=plot_config['main_plot'], data=data)
-
+    fig = add_areas(fig, 1, data, plot_config['main_plot'])
     fig = plot_trades(fig, trades)
-
-    # Volume goes to row 2
+    # sub plot: Volume goes to row 2
     volume = go.Bar(
         x=data['date'],
         y=data['volume'],
@@ -384,23 +429,26 @@ def generate_candlestick_graph(pair: str, data: pd.DataFrame, trades: pd.DataFra
         marker_line_color='DarkSlateGrey'
     )
     fig.add_trace(volume, 2, 1)
-
-    # Add indicators to separate row
-    for i, name in enumerate(plot_config['subplots']):
-        fig = add_indicators(fig=fig, row=3 + i,
-                             indicators=plot_config['subplots'][name],
+    # add each sub plot to a separate row
+    for i, label in enumerate(plot_config['subplots']):
+        sub_config = plot_config['subplots'][label]
+        row = 3 + i
+        fig = add_indicators(fig=fig, row=row, indicators=sub_config,
                              data=data)
-
+        # fill area between indicators ( 'fill_to': 'other_indicator')
+        fig = add_areas(fig, row, data, sub_config)
     return fig
 
 
 def generate_profit_graph(pairs: str, data: Dict[str, pd.DataFrame],
-                          trades: pd.DataFrame, timeframe: str) -> go.Figure:
+                          trades: pd.DataFrame, timeframe: str, stake_currency: str) -> go.Figure:
     # Combine close-values for all pairs, rename columns to "pair"
     df_comb = combine_dataframes_with_mean(data, "close")
 
     # Trim trades to available OHLCV data
     trades = extract_trades_of_period(df_comb, trades, date_index=True)
+    if len(trades) == 0:
+        raise OperationalException('No trades found in selected timerange.')
 
     # Add combined cumulative profit
     df_comb = create_cum_profit(df_comb, trades, 'cum_profit', timeframe)
@@ -418,8 +466,8 @@ def generate_profit_graph(pairs: str, data: Dict[str, pd.DataFrame],
                         subplot_titles=["AVG Close Price", "Combined Profit", "Profit per pair"])
     fig['layout'].update(title="Freqtrade Profit plot")
     fig['layout']['yaxis1'].update(title='Price')
-    fig['layout']['yaxis2'].update(title='Profit')
-    fig['layout']['yaxis3'].update(title='Profit')
+    fig['layout']['yaxis2'].update(title=f'Profit {stake_currency}')
+    fig['layout']['yaxis3'].update(title=f'Profit {stake_currency}')
     fig['layout']['xaxis']['rangeslider'].update(visible=False)
 
     fig.add_trace(avgclose, 1, 1)
@@ -482,7 +530,7 @@ def load_and_plot_trades(config: Dict[str, Any]):
 
     exchange = ExchangeResolver.load_exchange(config['exchange']['name'], config)
     IStrategy.dp = DataProvider(config, exchange)
-    plot_elements = init_plotscript(config, strategy.startup_candle_count)
+    plot_elements = init_plotscript(config, list(exchange.markets), strategy.startup_candle_count)
     timerange = plot_elements['timerange']
     trades = plot_elements['trades']
     pair_counter = 0
@@ -517,7 +565,8 @@ def plot_profit(config: Dict[str, Any]) -> None:
     But should be somewhat proportional, and therefor useful
     in helping out to find a good algorithm.
     """
-    plot_elements = init_plotscript(config)
+    exchange = ExchangeResolver.load_exchange(config['exchange']['name'], config)
+    plot_elements = init_plotscript(config, list(exchange.markets))
     trades = plot_elements['trades']
     # Filter trades to relevant pairs
     # Remove open pairs - we don't know the profit yet so can't calculate profit for these.
@@ -532,6 +581,7 @@ def plot_profit(config: Dict[str, Any]) -> None:
     # Create an average close price of all the pairs that were involved.
     # this could be useful to gauge the overall market trend
     fig = generate_profit_graph(plot_elements['pairs'], plot_elements['ohlcv'],
-                                trades, config.get('timeframe', '5m'))
+                                trades, config.get('timeframe', '5m'),
+                                config.get('stake_currency', ''))
     store_plot_file(fig, filename='freqtrade-profit-plot.html',
                     directory=config['user_data_dir'] / 'plot', auto_open=True)
